CREATE OR REPLACE PACKAGE BODY kopiointi_henkilo AS
/** Sislt HENKILO-taulun kopiointiajojen implementaatiot.
 *
 *  $Id: kopiointi_henkilo.pkb,v 1.3 2004/07/08 17:59:16 rrasanen Exp $
 */


/** Kopioi HENKILO-nkymn DAWA:sta SIIRTO_HENKILO-tauluun.
 *
 *  <p>
 *  Ei commitoi.
 *
 *  <p>
 *  @scope                              public
 */

PROCEDURE kopioi IS
BEGIN
  INSERT
  INTO siirto_henkilo
       (etunimi, sukunimi, henkilotunnus, kayttajatunnus, lahiosoite, postinumero, postitoimipaikka
       ,sahkopostiosoite, vakanssiryhma, s_vakanssiryhma)
       (SELECT etunimi, sukunimi, henkilotunnus, kayttajatunnus, lahiosoite, postinumero
             , postitoimipaikka, sahkopostiosoite, vakanssiryhma, s_vakanssiryhma
          FROM dawa_henkilo
       )
  ;
END kopioi;



/** Generoi HTUNNUS-kenttaan sopivan arvon...
 *
 */

FUNCTION generoi_htunnus (p_etunimi IN VARCHAR2
                         ,p_sukunimi IN VARCHAR2
                         ,p_max_pituus IN INTEGER DEFAULT 12
) RETURN VARCHAR2 IS
BEGIN
  RETURN (pkg_henkilo.generoi_htunnus (p_etunimi, p_sukunimi, p_max_pituus));
END generoi_htunnus;



/** Ksittelee siirtotaulun.
 *
 *  <p>
 *  HUOM. Kopiointiosajrjestelm ei koskaan poista rivej PROJEKTI-taulusta,
 *  eik mistn muustakaan taulusta, johon kopioidaan tietovarastosta tavaraa.
 *
 *  <p>
 *  Tm menisi NIIIIIN ntisti Oracle9:n MERGE-komennolla...
 *
 *  <p>
 *  @scope                              public
 */

PROCEDURE kasittele IS
  CURSOR crs_henkilo IS
    SELECT siihlo.etunimi AS siihlo_etunimi
         , siihlo.sukunimi AS siihlo_sukunimi
         , siihlo.henkilotunnus AS siihlo_henkilotunnus
         , siihlo.lahiosoite AS siihlo_lahiosoite
         , siihlo.postinumero AS siihlo_postinumero
         , siihlo.postitoimipaikka AS siihlo_postitoimipaikka
         , siihlo.sahkopostiosoite AS siihlo_sahkopostiosoite
         , henklo.htunnus AS henklo_htunnus
         , henklo.etunimet AS henklo_etunimet
         , henklo.sukunimi AS henklo_sukunimi
         , henklo.hetu AS henklo_hetu
         , henklo.katuosoite AS henklo_katuosoite
         , henklo.postinro AS henklo_postinro
         , henklo.postitoimipaikka AS henklo_postitoimipaikka
         , henklo.sahkopostiosoite AS henklo_sahkopostiosoite
      FROM henkilo henklo
         , siirto_henkilo siihlo
     WHERE henklo.hetu (+) = siihlo.henkilotunnus
    ;
  l_tiedot_muuttuneet   BOOLEAN;
  l_generoitu_htunnus   VARCHAR2 (4000);
BEGIN
  -- Loopataan kaikki tulosjoukon rivit CURSOR-FOR-LOOPissa.
  FOR c IN crs_henkilo LOOP

    -- Jos HENKLO.HETU on NULL, on SIIRTO_HENKILO-taulussa rivi, jota
    -- vastaavaa rivi ei ole HENKILO-taulussa ja tllin rivi listn.
    IF (c.henklo_hetu IS NULL) THEN
      -- Generoidaan HTUNNUS.
      l_generoitu_htunnus := generoi_htunnus (c.siihlo_etunimi, c.siihlo_sukunimi);
      
      INSERT
        INTO henkilo
             (htunnus, etunimet, sukunimi, hetu, katuosoite, postinro
             ,postitoimipaikka, sahkopostiosoite)
      VALUES (l_generoitu_htunnus
             ,NVL (c.siihlo_etunimi, '(ei tiedossa)')
             ,NVL (c.siihlo_sukunimi, '(ei tiedossa)')
             ,c.siihlo_henkilotunnus
             ,c.siihlo_lahiosoite
             ,c.siihlo_postinumero
             ,c.siihlo_postitoimipaikka
             ,c.siihlo_sahkopostiosoite
             )
      ;
    ELSE
      -- Muuttumisen tarkistus kommentoitu pois. Kaikki olemassaolevat rivit
      -- pivitetn Syy: NULL-kenttien tarkistus ei mene oikein, jos toinen
      -- arvoista on NULL. Pit verrata NVL()-funktiolla, jolloin toimii. Ei
      -- kuitenkaan toteutettu, koska ei kerettisi testaamaan.
      l_tiedot_muuttuneet := TRUE;
      
      /*
      -- Muussa tapauksessa tarkistetaan, pitk pivitt rivi ajan tasalle.
      -- Pivitys tehdn vain, jos rivin tiedot tauluissa PROJEKTI ja
      -- SIIRTO_PROJEKTI poikkeavat toisistaan jossain sarakkeessa. Nin vltetn
      -- turhat pivitykset ja mahdollisten triggereiden turha laukeaminen.
      l_tiedot_muuttuneet := FALSE;

      IF (c.henklo_etunimet != c.siihlo_etunimi) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.henklo_sukunimi != c.siihlo_sukunimi) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.henklo_katuosoite != c.siihlo_lahiosoite) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.henklo_postinro != c.siihlo_postinumero) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.henklo_postitoimipaikka != c.siihlo_postitoimipaikka) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.henklo_sahkopostiosoite != c.siihlo_sahkopostiosoite) THEN
        l_tiedot_muuttuneet := TRUE;
      END IF;
      */      
      IF (l_tiedot_muuttuneet) THEN
        UPDATE henkilo
           SET etunimet = NVL (c.siihlo_etunimi, '(ei tiedossa)')
             , sukunimi = NVL (c.siihlo_sukunimi, '(ei tiedossa)')
             , katuosoite = c.siihlo_lahiosoite
             , postinro = c.siihlo_postinumero
             , postitoimipaikka = c.siihlo_postitoimipaikka
             , sahkopostiosoite = c.siihlo_sahkopostiosoite
         WHERE hetu = c.siihlo_henkilotunnus
        ;
      END IF;
    END IF;
  END LOOP;
END kasittele;

END kopiointi_henkilo;
/
