CREATE OR REPLACE PACKAGE BODY kopiointi_projekti AS
/** Sislt PROJEKTI-taulun kopiointiajojen implementaatiot.
 *
 *  $Id: kopiointi_projekti.pkb,v 1.4 2004/07/08 17:59:16 rrasanen Exp $
 */



/** Kopioi PROJEKTI-nkymn DAWA:sta SIIRTO_PROJEKTI-tauluun.
 *
 *  <p>
 *  Ei commitoi.
 *
 *  <p>
 *  @scope                              public
 */

PROCEDURE kopioi IS
BEGIN
  INSERT
    INTO siirto_projekti
         (alkupaivamaara, loppupaivamaara, projekti, s_projekti
         ,projektivastaava)
         (SELECT alkupaivamaara, loppupaivamaara, projekti, s_projekti
               , projektivastaava
            FROM dawa_projekti
         )
  ;
END kopioi;



/** Ksittelee siirtotaulun.
 *
 *  <p>
 *  Ei commitoi.
 *
 *  <p>
 *  HUOM. Kopiointiosajrjestelm ei koskaan poista rivej PROJEKTI-taulusta,
 *  eik mistn muustakaan taulusta, johon kopioidaan tietovarastosta tavaraa.
 *
 *  <p>
 *  Tm menisi NIIIIIN ntisti Oracle9:n MERGE-komennolla...
 *
 *  <p>
 *  @scope                              public
 */

PROCEDURE kasittele IS
  -- Kursori, joka hakee SIIRTO_PROJEKTI-taulusta kaikki projektit ja tekee
  -- OUTER JOINin PROJEKTI tauluun. Tulosjoukkoon tulevat kaikki
  -- SIIRTO_PROJEKTI-taulun rivit. Mikli SIIRTO_PROJEKTI-taulun rivi vastaavaa
  -- rivi ei lydy PROJEKTI-taulusta, PROJEKTI-taulun sarakkeet tulevat
  -- tulosjoukkoon NULLeina. Tm tarkoittaa sit, ett DAWA:sta on tullut
  -- uusi projekti, jota ei viel ole PROJEKTI-taulussa.
  CURSOR crs_projekti IS
    SELECT siiprj.projekti AS siiprj_projekti
         , siiprj.s_projekti AS siiprj_s_projekti
         , siiprj.alkupaivamaara AS siiprj_alkupaivamaara
         , siiprj.loppupaivamaara AS siiprj_loppupaivamaara
         , siiprj.projektivastaava AS siiprj_projektivastaava
         , prjkti.koodi AS prjkti_koodi
         , prjkti.nimi AS prjkti_nimi
         , prjkti.alkupvm AS prjkti_alkupvm
         , prjkti.loppupvm AS prjkti_loppupvm
         , prjkti.vastuuhenkilo AS prjkti_vastuuhenkilo
      FROM projekti prjkti
         , siirto_projekti siiprj
     WHERE prjkti.koodi (+) = siiprj.projekti
  ;
  l_tiedot_muuttuneet   BOOLEAN;
BEGIN
  -- Loopataan kaikki tulosjoukon rivit CURSOR-FOR-LOOPissa.
  FOR c IN crs_projekti LOOP
  
    -- Jos PRJKTI.KOODI on NULL, on SIIRTO_PROJEKTI-taulussa rivi, jota
    -- vastaavaa rivi ei ole PROJEKTI-taulussa ja tllin rivi listn.
    IF (c.prjkti_koodi IS NULL) THEN
      INSERT
        INTO projekti (koodi, nimi, alkupvm, loppupvm, vastuuhenkilo)
      VALUES (c.siiprj_projekti, c.siiprj_s_projekti, c.siiprj_alkupaivamaara
             ,c.siiprj_loppupaivamaara, c.siiprj_projektivastaava)
      ;

    ELSE
      -- Muuttumisen tarkistus kommentoitu pois. Kaikki olemassaolevat rivit
      -- pivitetn Syy: NULL-kenttien tarkistus ei mene oikein, jos toinen
      -- arvoista on NULL. Pit verrata NVL()-funktiolla, jolloin toimii. Ei
      -- kuitenkaan toteutettu, koska ei kerettisi testaamaan.
      l_tiedot_muuttuneet := TRUE;
      
      /*
      -- Muussa tapauksessa tarkistetaan, pitk pivitt rivi ajan tasalle.
      -- Pivitys tehdn vain, jos rivin tiedot tauluissa PROJEKTI ja
      -- SIIRTO_PROJEKTI poikkeavat toisistaan jossain sarakkeessa. Nin vltetn
      -- turhat pivitykset ja mahdollisten triggereiden turha laukeaminen.
      l_tiedot_muuttuneet := FALSE;

      IF (c.prjkti_nimi != c.siiprj_s_projekti) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.prjkti_alkupvm != c.siiprj_alkupaivamaara) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.prjkti_loppupvm != c.siiprj_loppupaivamaara) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.prjkti_vastuuhenkilo != c.siiprj_projektivastaava) THEN
        l_tiedot_muuttuneet := TRUE;
      END IF; 
      */
      
      IF (l_tiedot_muuttuneet) THEN
        UPDATE projekti
           SET nimi = c.siiprj_s_projekti
             , alkupvm = c.siiprj_alkupaivamaara
             , loppupvm = c.siiprj_loppupaivamaara
             , vastuuhenkilo = c.siiprj_projektivastaava
         WHERE koodi = c.prjkti_koodi
        ;
      END IF;
    END IF;
  END LOOP;
END kasittele;




/** Lis dummy-projektin (placeholder). Jos SIIRTO_SOPIMUSJAKSO-taulusta lytyy
 *  projekteja, joita ei lydy PROJEKTI-taulusta, listn nille projekteille
 *  dummy-projektit.
 *
 *  <p>
 *  @param      p_koodi                 projektin koodi
 *  @scope                              public
 */

PROCEDURE lisaa_dummy (p_koodi IN projekti.koodi%TYPE) IS
BEGIN
  INSERT
    INTO projekti
         (koodi, nimi, alkupvm, loppupvm, vastuuhenkilo)
  VALUES (p_koodi
         ,'(ei tiedossa:dummy-projekti)'
         ,SYSDATE
         ,NULL
         ,'(ei tiedossa:dummy-projekti)'
         )
  ;
EXCEPTION
  -- Skipataan ORA-00001 exceptionit ("duplicate value on index"), eli jos
  -- projekti on jo olemassa.
  WHEN DUP_VAL_ON_INDEX THEN
    NULL;
END lisaa_dummy;

END kopiointi_projekti;
/
