CREATE OR REPLACE PACKAGE BODY kopiointi_sopimusjakso AS

/** Sislt TYOSOPIMUSJAKSO-taulun kopiointiajojen implementaatiot.
 *
 *  $Id: kopiointi_sopimusjakso.pkb,v 1.4 2004/07/08 17:59:16 rrasanen Exp $
 */



/** Tarkistaa, lytyyk kaikille SIIRTO_SOPIMUSJAKSO-taulun riveille, jotka
 *  liittyvt projektiin, rivi PROJEKTI-taulusta. Jos projektia ei lydy, lis
 *  dummy-projektin (kts. paketti KOPIOINTI_PROJEKTI).
 *
 *  <p>
 *  @scope                              private
 */

PROCEDURE tarkista_projektit IS
  -- Hakee niiden projektien koodit, jotka ovat SIIRTO_SOPIMUSJAKSO-taulussa,
  -- mutta eivt PROJEKTI-taulussa. MINUS-operaattori on kohtalaisen rivakka.
  CURSOR crs_projektit IS
    SELECT projekti AS prjkti_koodi
      FROM siirto_sopimusjakso
     WHERE projekti IS NOT NULL
   MINUS
    SELECT koodi AS prjkti_koodi
      FROM projekti
    ;
BEGIN
  FOR c IN crs_projektit LOOP
    kopiointi_projekti.lisaa_dummy (c.prjkti_koodi);
  END LOOP;
END tarkista_projektit;



/** Kopioi SOPIMUSJAKSO-nkymn DAWA:sta SIIRTO_SOPIMUSJAKSO-tauluun.
 *
 *  <p>
 *  Ei commitoi.
 *
 *  <p>
 *  @scope                              public
 */

PROCEDURE kopioi IS
BEGIN
  INSERT
    INTO siirto_sopimusjakso
         (sopimusnumero, alkupaivamaara_sopimus, loppupaivamaara_sopimus, alkupaivamaara_sopimusjakso
         ,loppupaivamaara_sopimusjakso, vv_hoitoprosentti, henkilotunnus, vakanssi,nimikekoodi
         ,nimike,projekti, s_projekti, virka_tyosuhde, sopimustyyppi, s_sopimustyyppi,tunnit)
         (SELECT sopimusnumero, alkupaivamaara_sopimus, loppupaivamaara_sopimus
               , alkupaivamaara_sopimusjakso, loppupaivamaara_sopimusjakso, vv_hoitoprosentti
               , henkilotunnus, vakanssi,nimikekoodi, nimike, projekti, s_projekti, virka_tyosuhde
               , sopimustyyppi, s_sopimustyyppi,tunnit
            FROM dawa_sopimusjakso
         )
  ;
END kopioi;



/** Ksittelee siirtotaulun.
 *
 *  <p>
 *  Ei commitoi.
 *
 *  <p>
 *  Tss oletetaan, ett SOPIMUSJAKSON primary key koostuu sarakkeista
 *  SOPIMUSNUMERO,
 *  HENKLO_HTUNNUS ja ALKUPVM_JAKSO.
 *
 *  <p>
 *  HUOM. Kopiointiosajrjestelm ei koskaan poista rivej SOPIMUSJAKSO-taulusta,
 *  eik mistn muustakaan taulusta, johon kopioidaan tietovarastosta tavaraa.
 *
 *  <p>
 *  Tm menisi NIIIIIN ntisti Oracle9:n MERGE-komennolla...
 *
 *  <p>
 *  @scope                              public
 */

PROCEDURE kasittele IS
  -- Kursori, joka hakee SIIRTO_SOPIMUSJAKSO-taulusta kaikki sopimusjaksot ja
  -- tekee OUTER JOINin TYOSOPIMUSJAKSO tauluun. Tulosjoukkoon tulevat kaikki
  -- SIIRTO_SOPIMUSJAKSO-taulun rivit. Mikli SIIRTO_SOPIMUSJAKSO-taulun rivi
  -- vastaavaa rivi ei lydy TYOSOPIMUSJAKSO-taulusta, TYOSOPIMUSJAKSO-taulun
  -- sarakkeet tulevat tulosjoukkoon NULLeina. Tm tarkoittaa sit, ett
  -- DAWA:sta on tullut uusi tysopimusjakso, jota ei viel ole
  -- TYOSOPIMUSJAKSO-taulussa.
  --
  -- Henkiln HTUNNUS haetaan inline-nkymss liitoksella HENKILO-tauluun.
  -- Samalla jvt pois ne mahdolliset SIIRTO_SOPIMUSJAKSO-taulun rivit, joille
  -- ei lydy henkil.
  --
  -- Todella ruma SQL, tekee kaksi outer-joinia... :/ Quick and dirty -ratkaisu. 
  CURSOR crs_sopimusjakso IS
    SELECT siisop.sopimusnumero AS siisop_sopimusnumero
         , siisop.htunnus AS siisop_henklo_htunnus
         , siisop.alkupaivamaara_sopimusjakso AS siisop_alkupvm_jakso
         , siisop.loppupaivamaara_sopimusjakso AS siisop_loppupvm_jakso
         , siisop.alkupaivamaara_sopimus AS siisop_alkupvm_sopimus
         , siisop.loppupaivamaara_sopimus AS siisop_loppupvm_sopimus
         , siisop.nimike AS siisop_nimike
         , siisop.virka_tyosuhde AS siisop_virka_tyosuhde
         , siisop.sopimustyyppi AS siisop_sopimustyyppi
         , siisop.s_sopimustyyppi AS siisop_s_sopimustyyppi
         , siisop.tunnit AS siisop_tunnit
         , siisop.vv_hoitoprosentti AS siisop_vv_hoitoprosentti
         , siisop.projekti AS siisop_prjkti_koodi
         , siisop.s_projekti AS siisop_prjkti_nimi
         , siisop.vakanssi AS siisop_vakanssi
         , siisop.nimikekoodi AS siisop_nimikekoodi
         , tsjaks.sopimusnumero AS tsjaks_sopimusnumero
         , tsjaks.loppupvm_jakso AS tsjaks_loppupvm_jakso
         , tsjaks.alkupvm_sopimus AS tsjaks_alkupvm_sopimus
         , tsjaks.loppupvm_sopimus AS tsjaks_loppupvm_sopimus
         , tsjaks.nimike AS tsjaks_nimike
         , tsjaks.virka_tyosuhde AS tsjaks_virka_tyosuhde
         , tsjaks.sopimustyyppi AS tsjaks_sopimustyyppi
         , tsjaks.s_sopimustyyppi AS tsjaks_s_sopimustyyppi
         , tsjaks.tunnit AS tsjaks_tunnit
         , tsjaks.vv_hoitoprosentti AS tsjaks_vv_hoitoprosentti
         , tsjaks.prjkti_koodi AS tsjaks_prjkti_koodi
         , tsjaks.prjkti_nimi AS tsjaks_prjkti_nimi
         , tsjaks.vakanssi AS tsjaks_vakanssi
         , tsjaks.nimikekoodi AS tsjaks_nimikekoodi
      FROM (SELECT s.sopimusnumero
                 , s.alkupaivamaara_sopimusjakso
                 , s.loppupaivamaara_sopimusjakso
                 , s.alkupaivamaara_sopimus
                 , s.loppupaivamaara_sopimus
                 , s.vv_hoitoprosentti
                 , h.htunnus
                 , s.vakanssi
                 , s.nimikekoodi
                 , s.nimike
                 , s.projekti
                 , s.s_projekti
                 , s.virka_tyosuhde
                 , s.sopimustyyppi
                 , s.s_sopimustyyppi
                 , s.tunnit
             FROM siirto_sopimusjakso s
                , henkilo h
                , projekti p
            WHERE s.henkilotunnus = h.hetu
              AND s.projekti = p.koodi (+)
           ) siisop
         , tyosopimusjakso tsjaks
     WHERE tsjaks.sopimusnumero (+) = siisop.sopimusnumero
       AND tsjaks.alkupvm_jakso (+) = siisop.alkupaivamaara_sopimusjakso
       AND tsjaks.henklo_htunnus (+) = siisop.htunnus
  ;
  l_tiedot_muuttuneet       BOOLEAN;
BEGIN
  -- Tarkistetaan, onko kaikille SIIRTO_SOPIMUSJAKSO-taulun projektikoodeille
  -- olemassa vastaava rivi PROJEKTI-taulussa. Lis jokaista puuttuvaa
  -- koodia kohde dummy-projektin.
  tarkista_projektit;

  -- Loopataan kaikki tulosjoukon rivit CURSOR-FOR-LOOPissa.
  FOR c IN crs_sopimusjakso LOOP
  
    -- Jos TYOSOPIMUSJAKSO.SOPIMUSNUMERO on NULL, on
    -- SIIRTO_SOPIMUSJAKSO-taulussa rivi, jota vastaavaa rivi ei ole
    -- TYOSOPIMUSJAKSO-taulussa ja tllin rivi listn.
    IF (c.tsjaks_sopimusnumero IS NULL) THEN
      INSERT
        INTO tyosopimusjakso
             (sopimusnumero, henklo_htunnus, alkupvm_jakso, loppupvm_jakso, alkupvm_sopimus
             ,loppupvm_sopimus, nimike, virka_tyosuhde, sopimustyyppi, s_sopimustyyppi, tunnit
             ,vv_hoitoprosentti, prjkti_koodi, prjkti_nimi, vakanssi, nimikekoodi)
      VALUES (c.siisop_sopimusnumero
             ,c.siisop_henklo_htunnus
             ,c.siisop_alkupvm_jakso
             ,c.siisop_loppupvm_jakso
             ,c.siisop_alkupvm_sopimus
             ,c.siisop_loppupvm_sopimus
             ,c.siisop_nimike
             ,c.siisop_virka_tyosuhde
             ,c.siisop_sopimustyyppi
             ,c.siisop_s_sopimustyyppi
             ,c.siisop_tunnit
             ,c.siisop_vv_hoitoprosentti
             ,c.siisop_prjkti_koodi
             ,c.siisop_prjkti_nimi
             ,c.siisop_vakanssi
             ,c.siisop_nimikekoodi
            )
      ;

    ELSE
      -- Muuttumisen tarkistus kommentoitu pois. Kaikki olemassaolevat rivit
      -- pivitetn Syy: NULL-kenttien tarkistus ei mene oikein, jos toinen
      -- arvoista on NULL. Pit verrata NVL()-funktiolla, jolloin toimii. Ei
      -- kuitenkaan toteutettu, koska ei kerettisi testaamaan.
      l_tiedot_muuttuneet := TRUE;
      
      /*
      -- Muussa tapauksessa tarkistetaan, pitk pivitt rivi ajan tasalle.
      -- Pivitys tehdn vain, jos rivin tiedot tauluissa TYOSOPIMUSJAKSO ja
      -- SIIRTO_SOPIMUSJAKSO poikkeavat toisistaan jossain sarakkeessa. Nin
      -- vltetn turhat pivitykset ja mahdollisten triggereiden turha
      -- laukeaminen.
      l_tiedot_muuttuneet := FALSE;

      IF (c.tsjaks_loppupvm_jakso != c.siisop_loppupvm_jakso) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_alkupvm_sopimus != c.siisop_alkupvm_sopimus) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_loppupvm_sopimus != c.siisop_loppupvm_sopimus) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_nimike != c.siisop_nimike) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_virka_tyosuhde != c.siisop_virka_tyosuhde) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_sopimustyyppi != c.siisop_sopimustyyppi) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_s_sopimustyyppi != c.siisop_s_sopimustyyppi) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_tunnit != c.siisop_tunnit) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_vv_hoitoprosentti != c.siisop_vv_hoitoprosentti) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_prjkti_koodi != c.siisop_prjkti_koodi) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_prjkti_nimi != c.siisop_prjkti_nimi) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_vakanssi != c.siisop_vakanssi) THEN
        l_tiedot_muuttuneet := TRUE;
      ELSIF (c.tsjaks_nimikekoodi != c.siisop_nimikekoodi) THEN
        l_tiedot_muuttuneet := TRUE;
      END IF;
      */
      IF (l_tiedot_muuttuneet) THEN
        UPDATE tyosopimusjakso
           SET loppupvm_jakso = c.siisop_loppupvm_jakso
             , alkupvm_sopimus = c.siisop_alkupvm_sopimus
             , loppupvm_sopimus = c.siisop_loppupvm_sopimus
             , nimike = c.siisop_nimike
             , virka_tyosuhde = c.siisop_virka_tyosuhde
             , sopimustyyppi = c.siisop_sopimustyyppi
             , s_sopimustyyppi = c.siisop_s_sopimustyyppi
             , tunnit = c.siisop_tunnit
             , vv_hoitoprosentti = c.siisop_vv_hoitoprosentti
             , prjkti_koodi = c.siisop_prjkti_koodi
             , prjkti_nimi = c.siisop_prjkti_nimi
             , vakanssi = c.siisop_vakanssi
             , nimikekoodi = c.siisop_nimikekoodi
         WHERE sopimusnumero = c.siisop_sopimusnumero
           AND henklo_htunnus = c.siisop_henklo_htunnus
           AND alkupvm_jakso = c.siisop_alkupvm_jakso
        ;
      END IF;
    END IF;
  END LOOP;
END kasittele;

END kopiointi_sopimusjakso;
/
