CREATE OR REPLACE PACKAGE BODY kopiointiosajarjestelma AS
/** Kopiointiosajrjestelmn "kyttliittymn" implementaatio.
 *
 *  $Id: kopiointiosajarjestelma.pkb,v 1.1 2004/06/24 19:00:08 rrasanen Exp $
 */


/** Tyhjent yhden taulun kyttmll TRUNCATE-ksky. Tehdn dynaamisella
 *  SQL:ll, koska PL/SQL:st ei voi kytt TRUNCATE-ksky.
 *
 *  <p>
 *  HUOM! Truncate ksky aiheuttaa implisiittisen COMMITin.
 *
 *  <p>
 *  @scope                              private
 *  @param      p_table_name            Taulun, nimi, johon ajetaan TRUNCATE.
 */

PROCEDURE truncate_table (p_table_name IN VARCHAR2) IS
  l_cursor                      INTEGER;
  l_sql                         VARCHAR2 (4000)         := 'TRUNCATE TABLE ';
  l_unused                      INTEGER;
BEGIN
  l_cursor := DBMS_SQL.OPEN_CURSOR;
  l_sql := l_sql || p_table_name;

  DBMS_SQL.PARSE (l_cursor, l_sql, DBMS_SQL.NATIVE);
  l_unused := DBMS_SQL.EXECUTE (l_cursor);
  DBMS_SQL.CLOSE_CURSOR (l_cursor);
EXCEPTION
  WHEN OTHERS THEN
    DBMS_SQL.CLOSE_CURSOR (l_cursor);
    RAISE;
END truncate_table;



/** Tyhjent kopioinnissa kytettvt siirto-taulut.
 *
 *  <p>
 *  HUOM! Tyhjennyksess kytetn TRUNCATE-komentoa, joka tekee
 *  implisiittisesti COMMITin.
 *
 *  <p>
 *  @scope      private
 */

PROCEDURE tyhjenna_siirto_taulut IS
BEGIN
  truncate_table ('siirto_henkilo');
  truncate_table ('siirto_projekti');
  truncate_table ('siirto_sopimusjakso');
END tyhjenna_siirto_taulut;



/** Kopioi datan DAWA:sta siirtotauluihin.
 *
 *  <p>
 *  Kopioinnin jlkeen tehdn COMMIT. Virheen sattuessa tehdn ROLLBACK ja
 *  heitetn exception ylemmlle tasolle.
 *
 *  <p>
 *  @scope                              private
 */

PROCEDURE kopioi_data_siirto_tauluihin IS
BEGIN
  kopiointi_henkilo.kopioi;
  kopiointi_projekti.kopioi;
  kopiointi_sopimusjakso.kopioi;

  COMMIT;
EXCEPTION
  WHEN OTHERS THEN
    ROLLBACK;
    RAISE;
END kopioi_data_siirto_tauluihin;



/** Ksittelee siirtotauluihin kopioidun datan pivittmll tarpeelliset
 *  tiedot varsinaisiin tauluihin.
 *
 *  <p>
 *  Virheen sattuessa tehdn ROLLBACK ja heitetn exception ylemmlle tasolle.
 *  Tllin siirtotauluihin j data, mutta se ei haittaa, koska siirtotaulut
 *  tyhjennetn seuraavan ajon alussa.
 *
 *  <p>
 *  @scope                              private
 */

PROCEDURE kasittele_kopioitu_data IS
BEGIN
  kopiointi_henkilo.kasittele;
  kopiointi_projekti.kasittele;
  kopiointi_sopimusjakso.kasittele;

  COMMIT;
EXCEPTION
  WHEN OTHERS THEN
    ROLLBACK;
    RAISE;
END kasittele_kopioitu_data;



/** Suorittaa koko kopiointi-ajo ketjun.
 *
 *  <p>
 *  @scope                              public
 */

PROCEDURE suorita IS
BEGIN
  suorita ('K');
END suorita;




/** Suorittaa koko kopiointi-ajo ketjun. Mitk vaiheet suoritetaan riippuu
 *  parametrin p_paivita_varsinaiset_taulut arvosta.
 *
 *  <p>
 *  @scope                              public
 *  @param      p_paivita_varsinaiset_taulut    Jos 'K', tekee kaikki vaiheet.
 *                                              Muutoin jtt ksittelyvaiheen
 *                                              tekemtt.
 */

PROCEDURE suorita (p_paivita_varsinaiset_taulut IN CHAR) IS
BEGIN
  tyhjenna_siirto_taulut;
  kopioi_data_siirto_tauluihin;

  IF (UPPER (p_paivita_varsinaiset_taulut) = 'K') THEN
    kasittele_kopioitu_data;
  END IF;
END suorita;

END kopiointiosajarjestelma;
/