CREATE OR REPLACE PACKAGE BODY pkg_henkilo AS
/** Sislt rutiineja HENKILO-taulun ksittelyyn. Kytnnss tllhetkell
 *  sislt vain HTUNNUS-kentn generointiin tarvittavat metodit.
 *
 *  $Id: pkg_henkilo.pkb,v 1.2 2004/07/05 18:28:41 rrasanen Exp $
 */

-- Privaatit muuttujat.

g_htunnus_oletus_etunimi        VARCHAR2(32)        := 'EITIETOA';
g_htunnus_oletus_sukunimi       VARCHAR2(32)        := 'EITIETOA';

/**
 *
 *  <p>
 *  @scope                              public
 *  @param      p_oletus                -
 */

PROCEDURE htunnus_aseta_oletus_etunimi (p_oletus IN VARCHAR2) IS
BEGIN
  g_htunnus_oletus_etunimi := p_oletus;
END htunnus_aseta_oletus_etunimi;

/**
 *
 *  <p>
 *  @scope                              public
 *  @param      p_oletus                -
 */

PROCEDURE htunnus_aseta_oletus_sukunimi (p_oletus IN VARCHAR2) IS
BEGIN
  g_htunnus_oletus_sukunimi := p_oletus;
END htunnus_aseta_oletus_sukunimi;



/** Tarkistaa, onko parametrina annettu htunnus jo olemassa.
 *
 *  <p>
 *  @scope                              public
 *  @param      p_htunnus               -
 *  @return                             -
 */

FUNCTION htunnus_olemassa (p_htunnus IN VARCHAR2)
RETURN BOOLEAN
IS
  CURSOR crs_tarkista_htunnus (p_htunnus IN VARCHAR2) IS
      SELECT 1
        FROM henkilo
       WHERE htunnus = p_htunnus
  ;

  l_result                  BOOLEAN             := FALSE;
  l_temp                    crs_tarkista_htunnus%ROWTYPE; 
BEGIN
  OPEN crs_tarkista_htunnus (p_htunnus);
  FETCH crs_tarkista_htunnus INTO l_temp;
  IF (crs_tarkista_htunnus%FOUND) THEN
    l_result := TRUE;
  END IF;
  CLOSE crs_tarkista_htunnus;

  RETURN (l_result);
END htunnus_olemassa;



/** Generoi yksilllisen htunnuksen uudelle henkillle.
 *
 *  <p>
 *  Pyrkii generoimaan htunnuksen "SUKUNIMI_E". Jos vastaava tunnus lytyy,
 *  generoi tunnukset seuraavasti:
 *    SUKUNIMI_E
 *    SUKUNIMI_ET
 *    SUKUNIM_E
 *    SUKUNIM_ET
 *    SUKUNIM_ETU
 *    SUKUNI_E
 *    SUKUNI_ET
 *    SUKUNI_ETU
 *    SUKUNI_ETUN
 *     ...
 *    S_ETUNIMI    
 *
 *  <p>
 *  Mikli tunnusta ei voida generoida, niin nostetaan EXCEPTION -20999.
 *
 *  <p>
 *  @scope                              public
 *  @param      p_etunimi               -
 *  @param      p_sukunimi              -
 *  @param      p_max_pituus            -
 *  @return                             -
 */

FUNCTION generoi_htunnus (p_etunimi IN VARCHAR2
                         ,p_sukunimi IN VARCHAR2
                         ,p_max_pituus IN INTEGER DEFAULT 12
)
RETURN VARCHAR2
IS
  l_merkkeja_sukunimeen     PLS_INTEGER         := p_max_pituus - 2;
  l_merkkeja_etunimeen      PLS_INTEGER         := 1;
  l_htunnus                 VARCHAR2(4000);
  l_etunimi                 VARCHAR2(4000)      := UPPER (REPLACE (p_etunimi, ' '));
  l_sukunimi                VARCHAR2(4000)      := UPPER (REPLACE (p_sukunimi, ' '));
  l_htunnus_olemassa        BOOLEAN             := TRUE;
BEGIN
  -- Tarkistetaan NULLit
  l_etunimi := NVL (l_etunimi, g_htunnus_oletus_etunimi);
  l_sukunimi := NVL (l_sukunimi, g_htunnus_oletus_sukunimi);

  -- Jos sukunimi on lyhyt...
  IF (LENGTH (l_sukunimi) < (p_max_pituus - 2)) THEN
    l_merkkeja_sukunimeen := LENGTH (l_sukunimi);
  END IF;

  -- Ulommainen looppi sukunimen pituuden suhteen...
  WHILE (l_merkkeja_sukunimeen > 0) LOOP

    -- Sisempi looppi etunimen pituuden suhteen...
    FOR i IN 1..LEAST (p_max_pituus - l_merkkeja_sukunimeen - 1, LENGTH (l_etunimi)) LOOP
      l_htunnus := SUBSTR (SUBSTR (l_sukunimi, 1, l_merkkeja_sukunimeen)
                          || '_' || SUBSTR (l_etunimi, 1, i)
                          , 1, p_max_pituus)
      ;
      l_htunnus_olemassa := htunnus_olemassa (l_htunnus);
      EXIT WHEN NOT l_htunnus_olemassa;
    END LOOP;
    EXIT WHEN NOT l_htunnus_olemassa;
    l_merkkeja_sukunimeen := l_merkkeja_sukunimeen - 1;
  END LOOP;

  IF (l_htunnus_olemassa) THEN
    RAISE_APPLICATION_ERROR (-20999, 'HTUNNUS:ta ei voitu generoida');
  END IF;

  RETURN (l_htunnus);
END generoi_htunnus;

END pkg_henkilo;
/
